@@ -19,7 +19,7 @@
 %                               January 1993                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -56,7 +56,7 @@
 %    -compose operator   composite operator
 %    -colorspace type    alternate image colorspace
 %    -comment string     annotate image with comment
-%    -compress type      None, JPEG, LZW, RunlengthEncoded, or Zip
+%    -compress type      type of image compression
 %    -density geometry   vertical and horizontal density of the image
 %    -displace geometry  shift image pixels as defined by a displacement map
 %    -display server     obtain image or font from this X server
@@ -64,16 +64,17 @@
 %    -dither             apply Floyd/Steinberg error diffusion to image
 %    -font name          X11 font for displaying text
 %    -geometry geometry  perferred size or location of the image
-%    -gravity direction  which direction to gravitate towards
+%    -gravity type       which direction to gravitate towards
 %    -interlace type     None, Line, Plane, or Partition
 %    -label name         assign a label to an image
 %    -matte              store matte channel if the image has one
 %    -monochrome         transform image to black and white
-%    -negate             apply color inversion to image
+%    -negate             replace every pixel with its complementary color 
 %    -page geometry      size and location of an image canvas
 %    -quality value      JPEG/MIFF/PNG compression level
 %    -scene value        image scene number
 %    -size geometry      width and height of image
+%    -stegano offset     hide watermark within an image
 %    -stereo             combine two image to create a stereo anaglyph
 %    -tile               repeat composite operation across image
 %    -treedepth value    depth of the color classification tree
@@ -85,8 +86,13 @@
 /*
   Include declarations.
 */
+#if !defined(macintosh)
+#include "magick/magick.h"
+#include "magick/defines.h"
+#else
 #include "magick.h"
-#include "version.h"
+#include "defines.h"
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -101,9 +107,9 @@
 %
 %  Procedure Usage displays the program usage;
 %
-%  The format of the Usage routine is:
+%  The format of the Usage method is:
 %
-%      Usage(client_name)
+%      void Usage(const char *client_name)
 %
 %    o client_name: a character string representing the name of the client
 %      program.
@@ -112,10 +118,10 @@
 */
 static void Usage(const char *client_name)
 {
-  char
+  const char
     **p;
 
-  static char
+  static const char
     *options[]=
     {
       "-blend value        blend the two images a given percent",
@@ -123,7 +129,7 @@ static void Usage(const char *client_name)
       "-colorspace type    alternate image colorspace",
       "-comment string     annotate image with comment",
       "-compose operator   composite operator",
-      "-compress type      None, JPEG, LZW, RunlengthEncoded, or Zip",
+      "-compress type      type of image compression",
       "-density geometry   vertical and horizontal density of the image",
       "-displace geometry  shift image pixels as defined by a displacement map",
       "-display server     obtain image or font from this X server",
@@ -131,16 +137,17 @@ static void Usage(const char *client_name)
       "-dither             apply Floyd/Steinberg error diffusion to image",
       "-font name          X11 font for displaying text",
       "-geometry geometry  perferred size or location of the image",
-      "-gravity direction  which direction to gravitate towards",
+      "-gravity type       which direction to gravitate towards",
       "-interlace type     None, Line, Plane, or Partition",
-      "-label name         assign a label to an image",
+      "-label name         ssign a label to an image",
       "-matte              store matte channel if the image has one",
       "-monochrome         transform image to black and white",
-      "-negate              apply color inversion to image",
+      "-negate             replace every pixel with its complementary color ",
       "-page geometry      size and location of an image canvas",
       "-quality value      JPEG/MIFF/PNG compression level",
       "-scene value        image scene number",
       "-size geometry      width and height of image",
+      "-stegano offset     hide watermark within an image",
       "-stereo             combine two image to create a stereo anaglyph",
       "-tile               repeat composite operation across image",
       "-treedepth value    depth of the color classification tree",
@@ -148,13 +155,13 @@ static void Usage(const char *client_name)
       (char *) NULL
     };
 
-  (void) printf("Version: %s\n",Version);
-  (void) printf("Copyright: %s\n\n",Copyright);
-  (void) printf("Usage: %s [options ...] image composite [mask] combined\n",
+  (void) printf("Version: %.1024s\n",MagickVersion);
+  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
+  (void) printf("Usage: %.1024s [options ...] image composite [mask] combined\n",
     client_name);
   (void) printf("\nWhere options include:\n");
   for (p=options; *p != (char *) NULL; p++)
-    (void) printf("  %s\n",*p);
+    (void) printf("  %.1024s\n",*p);
   Exit(0);
 }
 
@@ -187,7 +194,8 @@ int main(int argc,char **argv)
     compose;
 
   double
-    blend;
+    blend,
+    sans;
 
   Image
     *combined_image,
@@ -200,6 +208,7 @@ int main(int argc,char **argv)
 
   int
     gravity,
+    status,
     x,
     y;
 
@@ -207,18 +216,20 @@ int main(int argc,char **argv)
     i;
 
   unsigned int
+    stegano,
     stereo,
     tile;
 
   /*
-    Initialize program variables.
+    Initialize command line arguments.
   */
   ReadCommandlLine(argc,&argv);
   client_name=SetClientName(*argv);
+  (void) ExpandFilenames(&argc,&argv);
   if (argc < 4)
     Usage(client_name);
   /*
-    Read image and convert to MIFF format.
+    Set default.
   */
   blend=0.0;
   compose=ReplaceCompositeOp;
@@ -226,9 +237,10 @@ int main(int argc,char **argv)
   displacement_geometry=(char *) NULL;
   geometry=(char *) NULL;
   gravity=NorthWestGravity;
-  GetImageInfo(&image_info);
   image=(Image *) NULL;
+  GetImageInfo(&image_info);
   mask_image=(Image *) NULL;
+  stegano=0;
   stereo=False;
   tile=False;
   write_filename=argv[argc-1];
@@ -319,12 +331,16 @@ int main(int argc,char **argv)
                     image_info.colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
                     image_info.colorspace=RGBColorspace;
+                  if (Latin1Compare("srgb",option) == 0)
+                    image_info.colorspace=sRGBColorspace;
                   if (Latin1Compare("transparent",option) == 0)
                     image_info.colorspace=TransparentColorspace;
                   if (Latin1Compare("xyz",option) == 0)
                     image_info.colorspace=XYZColorspace;
                   if (Latin1Compare("ycbcr",option) == 0)
                     image_info.colorspace=YCbCrColorspace;
+                  if (Latin1Compare("ycc",option) == 0)
+                    image_info.colorspace=YCCColorspace;
                   if (Latin1Compare("yiq",option) == 0)
                     image_info.colorspace=YIQColorspace;
                   if (Latin1Compare("ypbpr",option) == 0)
@@ -405,6 +421,12 @@ int main(int argc,char **argv)
                   image_info.compression=UndefinedCompression;
                   if (Latin1Compare("None",option) == 0)
                     image_info.compression=NoCompression;
+                  if (Latin1Compare("BZip",option) == 0)
+                    image_info.compression=BZipCompression;
+                  if (Latin1Compare("Fax",option) == 0)
+                    image_info.compression=FaxCompression;
+                  if (Latin1Compare("Group4",option) == 0)
+                    image_info.compression=Group4Compression;
                   if (Latin1Compare("JPEG",option) == 0)
                     image_info.compression=JPEGCompression;
                   if (Latin1Compare("LZW",option) == 0)
@@ -431,24 +453,24 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info.density=argv[i];
+                  (void) CloneString(&image_info.density,argv[i]);
                 }
               break;
             }
-          if (strcmp("displace",option+1) == 0)
+          if (Latin1Compare("displace",option+1) == 0)
             {
               displacement_geometry=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing geometry",option);
-                  displacement_geometry=argv[i];
+                  (void) CloneString(&displacement_geometry,argv[i]);
                   compose=DisplaceCompositeOp;
                 }
               break;
             }
-          if (strcmp("display",option+1) == 0)
+          if (Latin1Compare("display",option+1) == 0)
             {
               image_info.server_name=(char *) NULL;
               if (*option == '-')
@@ -456,7 +478,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing server name",option);
-                  image_info.server_name=argv[i];
+                  (void) CloneString(&image_info.server_name,argv[i]);
                 }
               break;
             }
@@ -488,7 +510,7 @@ int main(int argc,char **argv)
               i++;
               if (i == argc)
                 MagickError(OptionError,"Missing font name",option);
-              image_info.font=argv[i];
+              (void) CloneString(&image_info.font,argv[i]);
             }
           break;
         }
@@ -502,20 +524,19 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  geometry=argv[i];
+                  (void) CloneString(&geometry,argv[i]);
                 }
               break;
             }
           if (strncmp("gravity",option+1,2) == 0)
             {
-              gravity=NorthWestGravity;
+              gravity=ForgetGravity;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  gravity=(-1);
                   if (Latin1Compare("Forget",option) == 0)
                     gravity=ForgetGravity;
                   if (Latin1Compare("NorthWest",option) == 0)
@@ -536,9 +557,7 @@ int main(int argc,char **argv)
                     gravity=SouthGravity;
                   if (Latin1Compare("SouthEast",option) == 0)
                     gravity=SouthEastGravity;
-                  if (Latin1Compare("Static",option) == 0)
-                    gravity=StaticGravity;
-                  if (gravity == (-1))
+                  if (gravity == ForgetGravity)
                     MagickError(OptionError,"Invalid gravity type",option);
                 }
               break;
@@ -636,7 +655,7 @@ int main(int argc,char **argv)
         {
           if (strncmp("quality",option+1,2) == 0)
             {
-              image_info.quality=atoi(DefaultImageQuality);
+              image_info.quality=75;
               if (*option == '-')
                 {
                   i++;
@@ -669,11 +688,23 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info.size=argv[i];
+                  (void) CloneString(&image_info.size,argv[i]);
                 }
               break;
             }
-          if (strncmp("stereo",option+1,2) == 0)
+          if (strncmp("stegano",option+1,4) == 0)
+            {
+              stegano=0;
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    MagickError(OptionError,"Missing offset",option);
+                  stegano=atoi(argv[i])+1;
+                }
+              break;
+            }
+          if (strncmp("stereo",option+1,4) == 0)
             {
               stereo=(*option == '-');
               break;
@@ -747,7 +778,7 @@ int main(int argc,char **argv)
       image->class=DirectClass;
       image->matte=True;
       p=image->pixels;
-      for (i=0; i < image->packets; i++)
+      for (i=0; i < (int) image->packets; i++)
       {
         p->index=index;
         p++;
@@ -756,7 +787,7 @@ int main(int argc,char **argv)
       composite_image->class=DirectClass;
       composite_image->matte=True;
       p=composite_image->pixels;
-      for (i=0; i < composite_image->packets; i++)
+      for (i=0; i < (int) composite_image->packets; i++)
       {
         p->index=index;
         p++;
@@ -767,98 +798,104 @@ int main(int argc,char **argv)
   /*
     Combine image.
   */
-  if (stereo)
-    combined_image=StereoImage(image,composite_image);
+  if (stegano != 0)
+    {
+      image->offset=stegano-1;
+      combined_image=SteganoImage(image,composite_image);
+    }
   else
-    if (tile)
-      {
-        /*
-          Tile the composite image.
-        */
-        for (y=0; y < image->rows; y+=composite_image->rows)
-          for (x=0; x < image->columns; x+=composite_image->columns)
-            CompositeImage(image,compose,composite_image,x,y);
-        combined_image=image;
-      }
+    if (stereo)
+      combined_image=StereoImage(image,composite_image);
     else
-      {
-        unsigned int
-          size;
-
-        /*
-          Digitally composite image.
-        */
-        size=0;
-        x=0;
-        y=0;
-        if (geometry != (char *) NULL)
-          (void) ParseImageGeometry(geometry,&x,&y,&size,&size);
-        switch (gravity)
+      if (tile)
         {
-          case NorthWestGravity:
-            break;
-          case NorthGravity:
-          {
-            x+=(image->columns-composite_image->columns) >> 1;
-            break;
-          }
-          case NorthEastGravity:
-          {
-            x+=image->columns-composite_image->columns;
-            break;
-          }
-          case WestGravity:
+          /*
+            Tile the composite image.
+          */
+          for (y=0; y < (int) image->rows; y+=composite_image->rows)
+            for (x=0; x < (int) image->columns; x+=composite_image->columns)
+              CompositeImage(image,compose,composite_image,x,y);
+          combined_image=image;
+        }
+      else
+        {
+          unsigned int
+            size;
+
+          /*
+            Digitally composite image.
+          */
+          size=0;
+          x=0;
+          y=0;
+          if (geometry != (char *) NULL)
+            (void) ParseImageGeometry(geometry,&x,&y,&size,&size);
+          switch (gravity)
           {
-            y+=(image->rows-composite_image->rows) >> 1;
-            break;
-          }
-          case ForgetGravity:
-          {
-            char
-              geometry[MaxTextExtent];
+            case NorthWestGravity:
+              break;
+            case NorthGravity:
+            {
+              x+=(image->columns-composite_image->columns) >> 1;
+              break;
+            }
+            case NorthEastGravity:
+            {
+              x+=image->columns-composite_image->columns;
+              break;
+            }
+            case WestGravity:
+            {
+              y+=(image->rows-composite_image->rows) >> 1;
+              break;
+            }
+            case ForgetGravity:
+            {
+              char
+                geometry[MaxTextExtent];
 
-            /*
-              Stretch composite to the same size as the image.
-            */
-            (void) sprintf(geometry,"%ux%u+0+0",image->columns,image->rows);
-            TransformImage(&composite_image,(char *) NULL,geometry);
-            break;
-          }
-          case StaticGravity:
-          case CenterGravity:
-          default:
-          {
-            x+=(image->columns-composite_image->columns) >> 1;
-            y+=(image->rows-composite_image->rows) >> 1;
-            break;
-          }
-          case EastGravity:
-          {
-            x+=image->columns-composite_image->columns;
-            y+=(image->rows-composite_image->rows) >> 1;
-            break;
-          }
-          case SouthWestGravity:
-          {
-            y+=image->rows-composite_image->rows;
-            break;
-          }
-          case SouthGravity:
-          {
-            x+=(image->columns-composite_image->columns) >> 1;
-            y+=image->rows-composite_image->rows;
-            break;
-          }
-          case SouthEastGravity:
-          {
-            x+=image->columns-composite_image->columns;
-            y+=image->rows-composite_image->rows;
-            break;
+              /*
+                Stretch composite to the same size as the image.
+              */
+              FormatString(geometry,"%ux%u+0+0",image->columns,image->rows);
+              TransformImage(&composite_image,(char *) NULL,geometry);
+              break;
+            }
+            case StaticGravity:
+            case CenterGravity:
+            default:
+            {
+              x+=(image->columns-composite_image->columns) >> 1;
+              y+=(image->rows-composite_image->rows) >> 1;
+              break;
+            }
+            case EastGravity:
+            {
+              x+=image->columns-composite_image->columns;
+              y+=(image->rows-composite_image->rows) >> 1;
+              break;
+            }
+            case SouthWestGravity:
+            {
+              y+=image->rows-composite_image->rows;
+              break;
+            }
+            case SouthGravity:
+            {
+              x+=(image->columns-composite_image->columns) >> 1;
+              y+=image->rows-composite_image->rows;
+              break;
+            }
+            case SouthEastGravity:
+            {
+              x+=image->columns-composite_image->columns;
+              y+=image->rows-composite_image->rows;
+              break;
+            }
           }
+          CompositeImage(image,compose,composite_image,x,y);
+          combined_image=image;
         }
-        CompositeImage(image,compose,composite_image,x,y);
-        combined_image=image;
-      }
   if (combined_image == (Image *) NULL)
     MagickError(OptionError,"Missing an image file name",(char *) NULL);
   /*
@@ -870,10 +907,11 @@ int main(int argc,char **argv)
     Write image.
   */
   (void) strcpy(combined_image->filename,write_filename);
-  (void) WriteImage(&image_info,combined_image);
+  SetImageInfo(&image_info,True);
+  status=WriteImage(&image_info,combined_image);
   if (image_info.verbose)
-    DescribeImage(combined_image,stderr,False);
-  DestroyImage(combined_image);
-  Exit(0);
+    DescribeImage(combined_image,stdout,False);
+  DestroyDelegateInfo();
+  Exit(status ? 0 : errno);
   return(False);
 }
