@@ -19,7 +19,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -59,13 +59,14 @@
 %  Usage: import [options ...] [file]
 %
 %  Where options include:
+%    -adjoin             join images into a single multi-image file
 %    -border             include image borders in the output image
 %    -colors value       preferred number of colors in the image
 %    -colorspace type    alternate image colorspace
 %    -comment string     annotate image with comment
-%    -compress type      None, JPEG, LZW, RunlengthEncoded, or Zip
+%    -compress type      type of image compression
 %    -crop geometry      preferred size and location of the cropped image
-%    -delay seconds      pause before selecting target window
+%    -delay value        pause before selecting target window
 %    -density geometry   vertical and horizontal density of the image
 %    -descend            obtain image by descending window hierarchy
 %    -display server     X server to contact
@@ -76,13 +77,13 @@
 %    -interlace type     None, Line, Plane, or Partition
 %    -label name         assign a label to an image
 %    -monochrome         transform image to black and white
-%    -negate             apply color inversion to image
+%    -negate             replace every pixel with its complementary color 
 %    -page geometry      size and location of an image canvas
 %    -ping               efficiently determine image width and height
 %    -pointsize value    pointsize of Postscript font
 %    -quality value      JPEG/MIFF/PNG compression level
 %    -rotate degrees     apply Paeth rotation to the image
-%    -scene value        image scene number
+%    -scene value        number of screen snapshots
 %    -screen             select image from root window
 %    -silent             operate silently, i.e. don't ring any bells 
 %    -transparent color  make this color transparent within the image
@@ -102,13 +103,15 @@
 /*
   Include declarations.
 */
+#if !defined(macintosh)
+#include "magick/magick.h"
+#include "magick/defines.h"
+#include "magick/xwindows.h"
+#else
 #include "magick.h"
-#include "version.h"
-
-/*
-  Define declarations.
-*/
-#define DefaultImportName  "magick.ps"
+#include "defines.h"
+#include "xwindows.h"
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -123,9 +126,9 @@
 %
 %  Procedure Usage displays the program usage;
 %
-%  The format of the Usage routine is:
+%  The format of the Usage method is:
 %
-%      Usage(client_name)
+%      void Usage(const char *client_name)
 %
 %    o client_name: a character string representing the name of the client
 %      program.
@@ -134,19 +137,20 @@
 */
 static void Usage(const char *client_name)
 {
-  char
+  const char
     **p;
 
-  static char
+  static const char
     *options[]=
     {
+      "-adjoin             join images into a single multi-image file",
       "-border             include image borders in the output image",
       "-colors value       preferred number of colors in the image",
       "-colorspace type    alternate image colorspace",
       "-comment string     annotate image with comment",
-      "-compress type      None, JPEG, LZW, RunlengthEncoded, or Zip",
+      "-compress type      type of image compression",
       "-crop geometry      preferred size and location of the cropped image",
-      "-delay seconds      pause before selecting target window",
+      "-delay value        display the next image after pausing",
       "-density geometry   vertical and horizontal density of the image",
       "-descend            obtain image by descending window hierarchy",
       "-display server     X server to contact",
@@ -157,13 +161,13 @@ static void Usage(const char *client_name)
       "-interlace type     None, Line, Plane, or Partition",
       "-label name         assign a label to an image",
       "-monochrome         transform image to black and white",
-      "-negate             apply color inversion to image",
+      "-negate             replace every pixel with its complementary color ",
       "-page geometry      size and location of an image canvas",
       "-ping               efficiently determine image width and height",
       "-pointsize value    pointsize of Postscript font",
       "-quality value      JPEG/MIFF/PNG compression level",
       "-rotate degrees     apply Paeth rotation to the image",
-      "-scene value        image scene number",
+      "-scene value        number of screen snapshots",
       "-treedepth value    depth of the color classification tree",
       "-transparent color  make this color transparent within the image",
       "-silent             operate silently, i.e. don't ring any bells ",
@@ -173,14 +177,14 @@ static void Usage(const char *client_name)
       (char *) NULL
     };
 
-  (void) printf("Version: %s\n",Version);
-  (void) printf("Copyright: %s\n\n",Copyright);
-  (void) printf("Usage: %s [options ...] [file]\n",client_name);
+  (void) printf("Version: %.1024s\n",MagickVersion);
+  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
+  (void) printf("Usage: %.1024s [options ...] [file]\n",client_name);
   (void) printf("\nWhere options include:\n");
   for (p=options; *p != (char *) NULL; p++)
-    (void) printf("  %s\n",*p);
+    (void) printf("  %.1024s\n",*p);
   (void) printf(
-  "\nBy default, 'file' is written in the Postscript image format.  To\n");
+  "\nBy default, 'file' is written in the MIFF image format.  To\n");
   (void) printf(
     "specify a particular image format, precede the filename with an image\n");
   (void) printf(
@@ -206,31 +210,38 @@ static void Usage(const char *client_name)
 */
 int main(int argc,char **argv)
 {
+#if defined(HasX11)
   char
     *client_name,
     *filename,
     *option,
     *resource_value,
+    *server_name,
     *target_window;
 
   Display
     *display;
 
   Image
-    *image;
+    *image,
+    *next_image;
 
   ImageInfo
-    image_info;
+    *image_info;
 
   int
     i,
+    status,
     x;
 
   QuantizeInfo
-    quantize_info;
+    *quantize_info;
+
+  register Image
+    *p;
 
   unsigned int
-    scene;
+    number_scenes;
 
   XImportInfo
     ximage_info;
@@ -242,16 +253,16 @@ int main(int argc,char **argv)
     resource_database;
 
   /*
-    Connect to X server.
+    Initialize command line arguments.
   */
-#if defined(WIN32)
-  SetErrorHandler(NTErrorHandler);
-  SetWarningHandler(NTWarningHandler);
-#endif
-  GetImageInfo(&image_info);
-  GetQuantizeInfo(&quantize_info);
-  ReadCommandlLine(argc,&argv);
+  SetNotifyHandlers;
   client_name=SetClientName(*argv);
+  ReadCommandlLine(argc,&argv);
+  (void) ExpandFilenames(&argc,&argv);
+  /*
+    Check for server name specified on the command line.
+  */
+  server_name=(char *) NULL;
   for (i=1; i < argc; i++)
   {
     /*
@@ -260,7 +271,7 @@ int main(int argc,char **argv)
     option=argv[i];
     if ((Extent(option) == 1) || ((*option != '-') && (*option != '+')))
       continue;
-    if (strcmp("display",option+1) == 0)
+    if (Latin1Compare("display",option+1) == 0)
       {
         /*
           User specified server name.
@@ -268,7 +279,7 @@ int main(int argc,char **argv)
         i++;
         if (i == argc)
           MagickError(OptionError,"Missing server name on -display",option);
-        image_info.server_name=argv[i];
+        server_name=argv[i];
         break;
       }
     if (strncmp("help",option+1,2) == 0)
@@ -277,19 +288,26 @@ int main(int argc,char **argv)
   /*
     Get user defaults from X resource database.
   */
-  display=XOpenDisplay(image_info.server_name);
+  display=XOpenDisplay(server_name);
   if (display == (Display *) NULL)
     MagickError(OptionError,"Unable to connect to X server",
-      XDisplayName(image_info.server_name));
+      XDisplayName(server_name));
   XSetErrorHandler(XError);
   resource_database=XGetResourceDatabase(display,client_name);
-  GetImageInfo(&image_info);
   XGetImportInfo(&ximage_info);
   XGetResourceInfo(resource_database,client_name,&resource_info);
+  image_info=resource_info.image_info;
+  quantize_info=resource_info.quantize_info;
   resource_value=
     XGetResourceInstance(resource_database,client_name,"border","False");
   ximage_info.borders=IsTrue(resource_value);
-  image_info.density=XGetResourceInstance(resource_database,client_name,
+  resource_info.delay=0;
+  resource_info.pause=0;
+  resource_value=
+    XGetResourceInstance(resource_database,client_name,"delay","6");
+  (void) XParseGeometry(resource_value,&x,&x,&resource_info.delay,
+    &resource_info.pause);
+  image_info->density=XGetResourceInstance(resource_database,client_name,
     "density",(char *) NULL);
   resource_value=
     XGetResourceInstance(resource_database,client_name,"descend","True");
@@ -299,25 +317,22 @@ int main(int argc,char **argv)
   ximage_info.frame=IsTrue(resource_value);
   resource_value=
     XGetResourceInstance(resource_database,client_name,"interlace","none");
-  image_info.interlace=UndefinedInterlace;
+  image_info->interlace=UndefinedInterlace;
   if (Latin1Compare("None",resource_value) == 0)
-    image_info.interlace=NoInterlace;
+    image_info->interlace=NoInterlace;
   if (Latin1Compare("Line",resource_value) == 0)
-    image_info.interlace=LineInterlace;
+    image_info->interlace=LineInterlace;
   if (Latin1Compare("Plane",resource_value) == 0)
-    image_info.interlace=PlaneInterlace;
+    image_info->interlace=PlaneInterlace;
   if (Latin1Compare("Partition",resource_value) == 0)
-    image_info.interlace=PartitionInterlace;
-  if (image_info.interlace == UndefinedInterlace)
+    image_info->interlace=PartitionInterlace;
+  if (image_info->interlace == UndefinedInterlace)
     MagickWarning(OptionWarning,"Unrecognized interlace type",resource_value);
-  image_info.page=XGetResourceInstance(resource_database,client_name,
+  image_info->page=XGetResourceInstance(resource_database,client_name,
     "pageGeometry",(char *) NULL);
   resource_value=
     XGetResourceInstance(resource_database,client_name,"quality","85");
-  image_info.quality=atoi(resource_value);
-  resource_value=
-    XGetResourceInstance(resource_database,client_name,"scene","0");
-  scene=atoi(resource_value);
+  image_info->quality=atoi(resource_value);
   resource_value=
     XGetResourceInstance(resource_database,client_name,"screen","False");
   ximage_info.screen=IsTrue(resource_value);
@@ -326,7 +341,11 @@ int main(int argc,char **argv)
   ximage_info.silent=IsTrue(resource_value);
   resource_value=
     XGetResourceInstance(resource_database,client_name,"verbose","False");
-  image_info.verbose=IsTrue(resource_value);
+  image_info->verbose=IsTrue(resource_value);
+  resource_value=
+    XGetResourceInstance(resource_database,client_name,"dither","True");
+  quantize_info->dither=IsTrue(resource_value);
+  number_scenes=1;
   /*
     Check command syntax.
   */
@@ -340,20 +359,32 @@ int main(int argc,char **argv)
     else
       switch(*(option+1))
       {
+        case 'a':
+        {
+          if (strncmp("adjoin",option+1,2) == 0)
+            {
+              image_info->adjoin=(*option == '-');
+              break;
+            }
+          MagickError(OptionError,"Unrecognized option",option);
+          break;
+        }
         case 'b':
         {
-          if (strcmp("border",option+1) == 0)
+          if (Latin1Compare("border",option+1) == 0)
             {
               ximage_info.borders=(*option == '-');
               break;
             }
           if (strncmp("bordercolor",option+1,7) == 0)
             {
-              if ((i == argc) || !sscanf(argv[i],"%d",&x))
+              image_info->border_color=(char *) NULL;
+              if (*option == '-')
                 {
                   i++;
                   if (i == argc)
-                    MagickError(OptionError,"Missing color",option);
+                    MagickError(OptionError,"Missing border color",option);
+                  (void) CloneString(&image_info->border_color,argv[i]);
                 }
               break;
             }
@@ -364,51 +395,55 @@ int main(int argc,char **argv)
         {
           if (strncmp("colors",option+1,7) == 0)
             {
-              quantize_info.number_colors=0;
+              quantize_info->number_colors=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing colors",option);
-                  quantize_info.number_colors=atoi(argv[i]);
+                  quantize_info->number_colors=atoi(argv[i]);
                 }
               break;
             }
           if (strncmp("colorspace",option+1,7) == 0)
             {
-              quantize_info.colorspace=RGBColorspace;
+              quantize_info->colorspace=RGBColorspace;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  quantize_info.colorspace=UndefinedColorspace;
+                  quantize_info->colorspace=UndefinedColorspace;
                   if (Latin1Compare("cmyk",option) == 0)
-                    quantize_info.colorspace=CMYKColorspace;
+                    quantize_info->colorspace=CMYKColorspace;
                   if (Latin1Compare("gray",option) == 0)
                     {
-                      quantize_info.colorspace=GRAYColorspace;
-                      quantize_info.number_colors=256;
-                      quantize_info.tree_depth=8;
+                      quantize_info->colorspace=GRAYColorspace;
+                      quantize_info->number_colors=256;
+                      quantize_info->tree_depth=8;
                     }
                   if (Latin1Compare("ohta",option) == 0)
-                    quantize_info.colorspace=OHTAColorspace;
+                    quantize_info->colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
-                    quantize_info.colorspace=RGBColorspace;
+                    quantize_info->colorspace=RGBColorspace;
+                  if (Latin1Compare("srgb",option) == 0)
+                    quantize_info->colorspace=sRGBColorspace;
                   if (Latin1Compare("transparent",option) == 0)
-                    quantize_info.colorspace=TransparentColorspace;
+                    quantize_info->colorspace=TransparentColorspace;
                   if (Latin1Compare("xyz",option) == 0)
-                    quantize_info.colorspace=XYZColorspace;
+                    quantize_info->colorspace=XYZColorspace;
                   if (Latin1Compare("ycbcr",option) == 0)
-                    quantize_info.colorspace=YCbCrColorspace;
+                    quantize_info->colorspace=YCbCrColorspace;
+                  if (Latin1Compare("ycc",option) == 0)
+                    quantize_info->colorspace=YCCColorspace;
                   if (Latin1Compare("yiq",option) == 0)
-                    quantize_info.colorspace=YIQColorspace;
+                    quantize_info->colorspace=YIQColorspace;
                   if (Latin1Compare("ypbpr",option) == 0)
-                    quantize_info.colorspace=YPbPrColorspace;
+                    quantize_info->colorspace=YPbPrColorspace;
                   if (Latin1Compare("yuv",option) == 0)
-                    quantize_info.colorspace=YUVColorspace;
-                  if (quantize_info.colorspace == UndefinedColorspace)
+                    quantize_info->colorspace=YUVColorspace;
+                  if (quantize_info->colorspace == UndefinedColorspace)
                     MagickError(OptionError,"Invalid colorspace type",option);
                 }
               break;
@@ -425,25 +460,31 @@ int main(int argc,char **argv)
             }
           if (strncmp("compress",option+1,5) == 0)
             {
-              image_info.compression=NoCompression;
+              image_info->compression=NoCompression;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.compression=UndefinedCompression;
+                  image_info->compression=UndefinedCompression;
                   if (Latin1Compare("None",option) == 0)
-                    image_info.compression=NoCompression;
+                    image_info->compression=NoCompression;
+                  if (Latin1Compare("BZip",option) == 0)
+                    image_info->compression=BZipCompression;
+                  if (Latin1Compare("Fax",option) == 0)
+                    image_info->compression=FaxCompression;
+                  if (Latin1Compare("Group4",option) == 0)
+                    image_info->compression=Group4Compression;
                   if (Latin1Compare("JPEG",option) == 0)
-                    image_info.compression=JPEGCompression;
+                    image_info->compression=JPEGCompression;
                   if (Latin1Compare("LZW",option) == 0)
-                    image_info.compression=LZWCompression;
+                    image_info->compression=LZWCompression;
                   if (Latin1Compare("RunlengthEncoded",option) == 0)
-                    image_info.compression=RunlengthEncodedCompression;
+                    image_info->compression=RunlengthEncodedCompression;
                   if (Latin1Compare("Zip",option) == 0)
-                    image_info.compression=ZipCompression;
-                  if (image_info.compression == UndefinedCompression)
+                    image_info->compression=ZipCompression;
+                  if (image_info->compression == UndefinedCompression)
                     MagickError(OptionError,"Invalid compression type",option);
                 }
               break;
@@ -465,25 +506,27 @@ int main(int argc,char **argv)
         {
           if (strncmp("delay",option+1,3) == 0)
             {
-              image_info.delay=(char *) NULL;
+              resource_info.delay=0;
+              resource_info.pause=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
-                    MagickError(OptionError,"Missing delay",option);
-                  image_info.delay=PostscriptGeometry(argv[i]);
+                    MagickError(OptionError,"Missing seconds",option);
+                  (void) XParseGeometry(argv[i],&x,&x,&resource_info.delay,
+                    &resource_info.pause);
                 }
               break;
             }
           if (strncmp("density",option+1,3) == 0)
             {
-              image_info.density=(char *) NULL;
+              image_info->density=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info.density=argv[i];
+                  (void) CloneString(&image_info->density,argv[i]);
                 }
               break;
             }
@@ -492,33 +535,33 @@ int main(int argc,char **argv)
               ximage_info.descend=(*option == '-');
               break;
             }
-          if (strcmp("display",option+1) == 0)
+          if (Latin1Compare("display",option+1) == 0)
             {
-              image_info.server_name=(char *) NULL;
+              image_info->server_name=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing server name",option);
-                  image_info.server_name=argv[i];
+                  (void) CloneString(&image_info->server_name,argv[i]);
                 }
               break;
             }
           if (strncmp("dispose",option+1,5) == 0)
             {
-              image_info.dispose=(char *) NULL;
+              image_info->dispose=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing method",option);
-                  image_info.dispose=PostscriptGeometry(argv[i]);
+                  image_info->dispose=PostscriptGeometry(argv[i]);
                 }
               break;
             }
           if (strncmp("dither",option+1,3) == 0)
             {
-              quantize_info.dither=(*option == '-');
+              quantize_info->dither=(*option == '-');
               break;
             }
           MagickError(OptionError,"Unrecognized option",option);
@@ -526,8 +569,13 @@ int main(int argc,char **argv)
         }
         case 'f':
         {
-          argv[i]="-ignore";  /* resolve option confict */
-          ximage_info.frame=(*option == '-');
+          if (strncmp("frame",option+1,2) == 0)
+            {
+              argv[i]="-ignore";  /* resolve option confict */
+              ximage_info.frame=(*option == '-');
+              break;
+            }
+          MagickError(OptionError,"Unrecognized option",option);
           break;
         }
         case 'g':
@@ -559,23 +607,23 @@ int main(int argc,char **argv)
         {
           if (strncmp("interlace",option+1,3) == 0)
             {
-              image_info.interlace=NoInterlace;
+              image_info->interlace=NoInterlace;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  image_info.interlace=UndefinedInterlace;
+                  image_info->interlace=UndefinedInterlace;
                   if (Latin1Compare("No",option) == 0)
-                    image_info.interlace=NoInterlace;
+                    image_info->interlace=NoInterlace;
                   if (Latin1Compare("Line",option) == 0)
-                    image_info.interlace=LineInterlace;
+                    image_info->interlace=LineInterlace;
                   if (Latin1Compare("Plane",option) == 0)
-                    image_info.interlace=PlaneInterlace;
+                    image_info->interlace=PlaneInterlace;
                   if (Latin1Compare("Partition",option) == 0)
-                    image_info.interlace=PartitionInterlace;
-                  if (image_info.interlace == UndefinedInterlace)
+                    image_info->interlace=PartitionInterlace;
+                  if (image_info->interlace == UndefinedInterlace)
                     MagickError(OptionError,"Invalid interlace type",option);
                 }
               break;
@@ -602,12 +650,12 @@ int main(int argc,char **argv)
         {
           if (strncmp("monochrome",option+1,2) == 0)
             {
-              image_info.monochrome=(*option == '-');
-              if (image_info.monochrome)
+              image_info->monochrome=(*option == '-');
+              if (image_info->monochrome)
                 {
-                  quantize_info.number_colors=2;
-                  quantize_info.tree_depth=8;
-                  quantize_info.colorspace=GRAYColorspace;
+                  quantize_info->number_colors=2;
+                  quantize_info->tree_depth=8;
+                  quantize_info->colorspace=GRAYColorspace;
                 }
               break;
             }
@@ -623,30 +671,30 @@ int main(int argc,char **argv)
         {
           if (strncmp("page",option+1,3) == 0)
             {
-              image_info.page=(char *) NULL;
+              image_info->page=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing page geometry",option);
-                  image_info.page=PostscriptGeometry(argv[i]);
+                  image_info->page=PostscriptGeometry(argv[i]);
                 }
               break;
             }
           if (strncmp("ping",option+1,2) == 0)
             {
-              image_info.ping=(*option == '-');
+              image_info->ping=(*option == '-');
               break;
             }
           if (strncmp("pointsize",option+1,2) == 0)
             {
-              image_info.pointsize=atoi(DefaultPointSize);
+              image_info->pointsize=12;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing size",option);
-                  image_info.pointsize=atoi(argv[i]);
+                  image_info->pointsize=atoi(argv[i]);
                 }
               break;
             }
@@ -657,13 +705,13 @@ int main(int argc,char **argv)
         {
           if (strncmp("quality",option+1,2) == 0)
             {
-              image_info.quality=0;
+              image_info->quality=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing quality",option);
-                  image_info.quality=atoi(argv[i]);
+                  image_info->quality=atoi(argv[i]);
                 }
               break;
             }
@@ -692,7 +740,7 @@ int main(int argc,char **argv)
               i++;
               if ((i == argc) || !sscanf(argv[i],"%d",&x))
                 MagickError(OptionError,"Missing scene",option);
-              scene=atoi(argv[i]);
+              number_scenes=atoi(argv[i]);
               break;
             }
           if (strncmp("screen",option+1,3) == 0)
@@ -722,13 +770,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("treedepth",option+1,3) == 0)
             {
-              quantize_info.tree_depth=0;
+              quantize_info->tree_depth=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing depth",option);
-                  quantize_info.tree_depth=atoi(argv[i]);
+                  quantize_info->tree_depth=atoi(argv[i]);
                 }
               break;
             }
@@ -740,12 +788,12 @@ int main(int argc,char **argv)
           i++;
           if (i == argc)
             MagickError(OptionError,"Missing id, name, or 'root'",option);
-          target_window=argv[i];
+          (void) CloneString(&target_window,argv[i]);
           break;
         }
         case 'v':
         {
-          image_info.verbose=(*option == '-');
+          image_info->verbose=(*option == '-');
           break;
         }
         case '?':
@@ -761,35 +809,49 @@ int main(int argc,char **argv)
       }
   }
   if (filename == (char *) NULL)
-    filename=DefaultImportName;
+    filename="magick.miff";
   /*
     Read image from X server.
   */
   if (target_window != (char *) NULL)
-    (void) strcpy(image_info.filename,target_window);
-  if (image_info.delay != (char *) NULL)
-    (void) sleep(atoi(image_info.delay));
-  image_info.colorspace=quantize_info.colorspace;
-  image_info.dither=quantize_info.dither;
-  image=ReadXImage(&image_info,&ximage_info);
+    (void) strcpy(image_info->filename,target_window);
+  (void) sleep(resource_info.pause);
+  image_info->colorspace=quantize_info->colorspace;
+  image_info->dither=quantize_info->dither;
+  image=(Image *) NULL;
+  for (i=0; i < (int) Max(number_scenes,1); i++)
+  {
+    next_image=XImportImage(image_info,&ximage_info);
+    if (next_image == (Image *) NULL)
+      continue;
+    (void) strcpy(next_image->filename,filename);
+    (void) strcpy(next_image->magick,"PS");
+    next_image->scene=i;
+    next_image->previous=image;
+    if (image != (Image *) NULL)
+      image->next=next_image;
+    image=next_image;
+    XDelay(display,(unsigned long) resource_info.delay*10);
+  }
   if (image == (Image *) NULL)
     MagickError(OptionError,"Missing an image file name",(char *) NULL);
-  (void) strcpy(image->magick,"X");
+  while (image->previous != (Image *) NULL)
+    image=image->previous;
   /*
     Transmogrify image as defined by the image processing options.
   */
-  MogrifyImage(&image_info,argc,argv,&image);
-  /*
-    Write import X image.
-  */
-  image->scene=scene;
-  (void) strcpy(image_info.filename,filename);
-  (void) strcpy(image->magick,"PS");
-  (void) strcpy(image->filename,filename);
-  (void) WriteImage(&image_info,image);
-  if (image_info.verbose)
-    DescribeImage(image,stderr,False);
-  DestroyImage(image);
-  Exit(0);
+  MogrifyImages(image_info,argc,argv,&image);
+  SetImageInfo(image_info,True);
+  for (p=image; p != (Image *) NULL; p=p->next)
+  {
+    status=WriteImage(image_info,p);
+    if ((status == False) || image_info->adjoin)
+      break;
+  }
+  if (image_info->verbose)
+    DescribeImage(image,stdout,False);
+  DestroyDelegateInfo();
+  Exit(status ? 0 : errno);
+#endif
   return(False);
 }
