@@ -9,8 +9,7 @@
 %        SSSSS  IIIII   GGG   N   N  A   A    T     UUU   R  R   EEEEE        %
 %                                                                             %
 %                                                                             %
-%                   Compute a Digital Signature for an Image                  %
-%                                                                             %
+%             Methods to Compute a Digital Signature for an Image             %
 %                                                                             %
 %                                                                             %
 %                             Software Design                                 %
@@ -18,7 +17,7 @@
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -46,10 +45,9 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  Routine SignatureImage computes a digital signature from the image
-%  colormap.  This signature uniquely identifies the colormap and is convenient
-%  for determining if the colormap of a sequence of images is identical when
-%  animating.  The digital signature is from RSA Data Security MD5 Digest
-%  Algorithm described in Internet draft [MD5], July 1992.
+%  pixels.  This signature uniquely identifies the image.  The digital
+%  signature is from RSA Data Security MD5 Digest Algorithm described in
+%  Internet draft [MD5], July 1992.
 %
 %
 */
@@ -58,6 +56,7 @@
   Include declarations.
 */
 #include "magick.h"
+#include "defines.h"
 
 /*
   Typedef declarations.
@@ -74,11 +73,12 @@ typedef struct _MessageDigest
 } MessageDigest;
 
 /*
-  Function prototypes.
+  Method prototypes.
 */
 static void
   TransformMessageDigest(MessageDigest *,unsigned long *),
-  UpdateMessageDigest(MessageDigest *,unsigned char *,unsigned long);
+  UpdateMessageDigest(MessageDigest *,const unsigned char *,
+    const unsigned long);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -91,11 +91,11 @@ static void
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function ComputeMessageDigest computes the message digest.
+%  Method ComputeMessageDigest computes the message digest.
 %
-%  The format of the ComputeMessageDigest routine is:
+%  The format of the ComputeMessageDigest method is:
 %
-%      ComputeMessageDigest(message_digest,input_message,message_length)
+%      void SignatureImage(Image *image)
 %
 %  A description of each parameter follows:
 %
@@ -174,9 +174,9 @@ static void ComputeMessageDigest(MessageDigest *message_digest)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function InitializeMessageDigest initializes the message digest structure.
+%  Method InitializeMessageDigest initializes the message digest structure.
 %
-%  The format of the InitializeMessageDigest routine is:
+%  The format of the InitializeMessageDigest method is:
 %
 %      InitializeMessageDigest(message_digest)
 %
@@ -210,9 +210,9 @@ static void InitializeMessageDigest(MessageDigest *message_digest)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function TransformMessageDigest updates the message digest.
+%  Method TransformMessageDigest updates the message digest.
 %
-%  The format of the TransformMessageDigest routine is:
+%  The format of the TransformMessageDigest method is:
 %
 %      TransformMessageDigest(message_digest,message)
 %
@@ -231,7 +231,7 @@ static void TransformMessageDigest(MessageDigest *message_digest,
 #define I(x,y,z)  ((y) ^ ((x) | (~z)))
 #define RotateLeft(x,n)  (((x) << (n)) | (((x) & 0xffffffff) >> (32-(n))))
 
-  static unsigned long
+  static const unsigned long
     additive_constant[64]=  /* 4294967296*abs(sin(i)), i in radians */
     {
       0xd76aa478, 0xe8c7b756, 0x242070db, 0xc1bdceee, 0xf57c0faf,
@@ -259,7 +259,9 @@ static void TransformMessageDigest(MessageDigest *message_digest,
     a,
     b,
     c,
-    d,
+    d;
+
+  register const unsigned long
     *p;
 
   /*
@@ -366,9 +368,9 @@ static void TransformMessageDigest(MessageDigest *message_digest,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function UpdateMessageDigest updates the message digest.
+%  Method UpdateMessageDigest updates the message digest.
 %
-%  The format of the UpdateMessageDigest routine is:
+%  The format of the UpdateMessageDigest method is:
 %
 %      UpdateMessageDigest(message_digest,input_message,message_length)
 %
@@ -379,33 +381,30 @@ static void TransformMessageDigest(MessageDigest *message_digest,
 %
 */
 static void UpdateMessageDigest(MessageDigest *message_digest,
-  unsigned char *input_message,unsigned long message_length)
+  const unsigned char *input_message,const unsigned long message_length)
 {
-  int
-    number_bytes;
-
   register unsigned char
     *p;
 
   register unsigned int
-    i;
+    i,
+    j;
 
   unsigned long
-    message[16];
+    message[16],
+    number_bits,
+    number_bytes;
 
   /*
-    Compute number of bytes mod 64.
+    Compute number of bits and bytes.
   */
-  number_bytes=(int) ((message_digest->number_bits[0] >> 3) & 0x3F);
-  /*
-    Update number of bits.
-  */
-  if (((message_digest->number_bits[0]+(message_length << 3)) & 0xffffffff) <
-      message_digest->number_bits[0])
+  number_bytes=(long) ((message_digest->number_bits[0] >> 3) & 0x3F);
+  number_bits=message_digest->number_bits[0]+(message_length << 3);
+  if ((number_bits & 0xffffffff) < message_digest->number_bits[0])
     message_digest->number_bits[1]++;
   message_digest->number_bits[0]+=message_length << 3;
   message_digest->number_bits[1]+=message_length >> 29;
-  while (message_length--)
+  for (i=0; i < message_length; i++)
   {
     /*
       Add new character to message.
@@ -417,12 +416,12 @@ static void UpdateMessageDigest(MessageDigest *message_digest,
           Transform message digest 64 bytes at a time.
         */
         p=message_digest->message;
-        for (i=0; i < 16; i++)
+        for (j=0; j < 16; j++)
         {
-          message[i]=(unsigned long) (*p++);
-          message[i]|=((unsigned long) (*p++)) << 8;
-          message[i]|=((unsigned long) (*p++)) << 16;
-          message[i]|=((unsigned long) (*p++)) << 24;
+          message[j]=(unsigned long) (*p++);
+          message[j]|=((unsigned long) (*p++)) << 8;
+          message[j]|=((unsigned long) (*p++)) << 16;
+          message[j]|=((unsigned long) (*p++)) << 24;
         }
         TransformMessageDigest(message_digest,message);
         number_bytes=0;
@@ -441,13 +440,13 @@ static void UpdateMessageDigest(MessageDigest *message_digest,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function SignatureImage computes a digital signature from an image.  This
+%  Method SignatureImage computes a digital signature from an image.  This
 %  signature uniquely identifies the image and is convenient for determining
 %  if the colormap of a sequence of images is identical when animating.  The
 %  digital signature is from RSA Data Security MD5 Digest Algorithm described
 %  in Internet draft [MD5], July 1992.
 %
-%  The format of the SignatureImage routine is:
+%  The format of the SignatureImage method is:
 %
 %      SignatureImage(image)
 %
@@ -460,8 +459,11 @@ static void UpdateMessageDigest(MessageDigest *message_digest,
 */
 Export void SignatureImage(Image *image)
 {
-  char
-    *hex;
+  const char
+    hex[] = "0123456789abcdef";
+
+  int
+    x;
 
   MessageDigest
     message_digest;
@@ -489,9 +491,10 @@ Export void SignatureImage(Image *image)
     Allocate memory for digital signature.
   */
   if (image->signature != (char *) NULL)
-    free((char *) image->signature);
-  image->signature=(char *) malloc(33*sizeof(char));
-  message=(unsigned char *) malloc(sizeof(RunlengthPacket));
+    FreeMemory((char *) image->signature);
+  image->signature=(char *) AllocateMemory(33*sizeof(char));
+  message=(unsigned char *)
+    AllocateMemory(image->columns*sizeof(RunlengthPacket));
   if ((image->signature == (char *) NULL) ||
       (message == (unsigned char *) NULL))
     {
@@ -503,25 +506,34 @@ Export void SignatureImage(Image *image)
     Compute image digital signature.
   */
   InitializeMessageDigest(&message_digest);
+  x=0;
   p=image->pixels;
-  for (i=0; i < image->packets; i++)
+  q=message;
+  for (i=0; i < (int) image->packets; i++)
   {
-    q=message;
-    WriteQuantum(p->red,q);
-    WriteQuantum(p->green,q);
-    WriteQuantum(p->blue,q);
-    WriteQuantum(p->index,q);
     for (j=0; j <= ((int) p->length); j++)
-      UpdateMessageDigest(&message_digest,message,q-message);
+    {
+      WriteQuantum(p->red,q);
+      WriteQuantum(p->green,q);
+      WriteQuantum(p->blue,q);
+      if (image->matte)
+        WriteQuantum(p->index,q);
+      x++;
+      if (x == (int) image->columns)
+        {
+          UpdateMessageDigest(&message_digest,message,q-message);
+          q=message;
+          x=0;
+        }
+    }
     p++;
   }
-  ComputeMessageDigest(&message_digest);
-  free((char *) message);
+  FreeMemory((char *) message);
   /*
     Convert digital signature to a 32 character hex string.
   */
+  ComputeMessageDigest(&message_digest);
   q=(unsigned char *) image->signature;
-  hex="0123456789abcdef";
   for (i=0; i < 16; i++)
   {
     *q++=hex[(message_digest.digest[i] >> 4) & 0xf];
