@@ -19,7 +19,7 @@
 %                            December 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1998 E. I. du Pont de Nemours and Company                        %
+%  Copyright 1999 E. I. du Pont de Nemours and Company                        %
 %                                                                             %
 %  Permission is hereby granted, free of charge, to any person obtaining a    %
 %  copy of this software and associated documentation files ("ImageMagick"),  %
@@ -54,20 +54,23 @@
 %  Usage: mogrify [options ...] file [ [options ...] file ...]
 %
 %  Where options include:
+%    -antialias           remove pixel-aliasing
 %    -blur factor         apply a filter to blur the image
 %    -border geometry     surround image with a border of color
+%    -bordercolor color   border color
 %    -box color           color for annotation bounding box
 %    -charcoal factor     simulate a charcoal drawing
 %    -colorize value      colorize the image with the pen color
 %    -colors value        preferred number of colors in the image
 %    -colorspace type     alternate image colorspace
 %    -comment string      annotate image with comment
-%    -compress type       None, JPEG, LZW, RunlengthEncoded, or Zip
+%    -compress type       type of image compression
 %    -contrast            enhance or reduce the image contrast
 %    -crop geometry       preferred size and location of the cropped image
 %    -cycle amount        cycle the image colormap
 %    -delay value         display the next image after pausing
 %    -density geometry    vertical and horizontal density of the image
+%    -depth value         depth of the image
 %    -despeckle           reduce the speckles within an image",
 %    -display server      obtain image or font from this X server
 %    -dispose method      GIF disposal method
@@ -77,11 +80,13 @@
 %    -emboss              emboss an image
 %    -enhance             apply a digital filter to enhance a noisy image
 %    -equalize            perform histogram equalization to an image
+%    -filter type         use this filter when resizing an image
 %    -flip                flip image in the vertical direction
 %    -flop                flop image in the horizontal direction
 %    -font name           X11 font for displaying text
 %    -format type         image format type
 %    -frame geometry      surround image with an ornamental border
+%    -fuzz distance       colors within this distance are considered equal
 %    -gamma value         level of gamma correction
 %    -geometry geometry   perferred size or location of the image
 %    -implode amount      implode image pixels about the center
@@ -94,7 +99,7 @@
 %    -matte               store matte channel if the image has one
 %    -modulate value      vary the brightness, saturation, and hue
 %    -monochrome          transform image to black and white
-%    -negate              apply color inversion to image
+%    -negate              replace every pixel with its complementary color 
 %    -noise               add or reduce noise in an image
 %    -normalize           transform image to span the full range of colors
 %    -opaque color        change this color to the pen color
@@ -115,14 +120,14 @@
 %    -sharpen factor      apply a filter to sharpen the image
 %    -shear geometry      slide one edge of the image along the X or Y axis
 %    -size geometry       width and height of image
-%    -solarize thresold   negate all pixels above the threshold level
+%    -solarize threshold   negate all pixels above the threshold level
 %    -spread amount       displace image pixels by a random amount
 %    -swirl degrees       swirl image pixels about the center
 %    -texture filename    name of texture to tile onto the image background
 %    -threshold value     threshold the image
 %    -transparent color   make this color transparent within the image
 %    -treedepth value     depth of the color classification tree
-%    -undercolor geometry control undercolor removal and black generation
+%    -units type          PixelsPerInch, PixelsPerCentimeter, or Undefined
 %    -verbose             print detailed information about the image
 %    -view                FlashPix viewing transforms
 %    -wave geometry       alter an image along a sine wave
@@ -139,8 +144,13 @@
 /*
   Include declarations.
 */
+#if !defined(macintosh)
+#include "magick/magick.h"
+#include "magick/defines.h"
+#else
 #include "magick.h"
-#include "version.h"
+#include "defines.h"
+#endif
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -153,11 +163,11 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Function Usage displays the program command syntax.
+%  Method Usage displays the program command syntax.
 %
-%  The format of the Usage routine is:
+%  The format of the Usage method is:
 %
-%      Usage(client_name)
+%      void Usage(const char *client_name)
 %
 %    o client_name: a character string representing the name of the client
 %      program.
@@ -166,26 +176,29 @@
 */
 static void Usage(const char *client_name)
 {
-  char
+  const char
     **p;
 
-  static char
+  static const char
     *options[]=
     {
+      "-antialias           remove pixel-aliasing",
       "-blur factor         apply a filter to blur the image",
       "-border geometry     surround image with a border of color",
+      "-bordercolor color   border color",
       "-box color           color for annotation bounding box",
       "-charcoal factor     simulate a charcoal drawing",
       "-colorize value      colorize the image with the pen color",
       "-colors value        preferred number of colors in the image",
       "-colorspace type     alternate image colorspace",
       "-comment string      annotate image with comment",
-      "-compress type       None, JPEG, LZW, RunlengthEncoded, or Zip",
+      "-compress type       type of image compression",
       "-contrast            enhance or reduce the image contrast",
       "-crop geometry       preferred size and location of the cropped image",
       "-cycle amount        cycle the image colormap",
       "-delay value         display the next image after pausing",
       "-density geometry    vertical and horizontal density of the image",
+      "-depth value         depth of the image",
       "-despeckle           reduce the speckles within an image",
       "-display server      obtain image or font from this X server",
       "-dispose method      GIF disposal method",
@@ -195,11 +208,13 @@ static void Usage(const char *client_name)
       "-emboss              emboss an image",
       "-enhance             apply a digital filter to enhance a noisy image",
       "-equalize            perform histogram equalization to an image",
+      "-filter type         use this filter when resizing an image",
       "-flip                flip image in the vertical direction",
       "-flop                flop image in the horizontal direction",
       "-font name           X11 font for displaying text",
       "-format type         image format type",
       "-frame geometry      surround image with an ornamental border",
+      "-fuzz distance       colors within this distance are considered equal",
       "-gamma value         level of gamma correction",
       "-geometry geometry   perferred size or location of the image",
       "-implode amount      implode image pixels about the center",
@@ -212,7 +227,7 @@ static void Usage(const char *client_name)
       "-matte               store matte channel if the image has one",
       "-modulate value      vary the brightness, saturation, and hue",
       "-monochrome          transform image to black and white",
-      "-negate              apply color inversion to image",
+      "-negate              replace every pixel with its complementary color ",
       "-noise               add or reduce noise in an image.",
       "-normalize           transform image to span the full range of colors",
       "-opaque color        change this color to the pen color",
@@ -233,27 +248,28 @@ static void Usage(const char *client_name)
       "-sharpen factor      apply a filter to sharpen the image",
       "-shear geometry      slide one edge of the image along the X or Y axis",
       "-size geometry       width and height of image",
-      "-solarize thresold   negate all pixels above the threshold level",
+      "-solarize threshold   negate all pixels above the threshold level",
       "-spread amount       displace image pixels by a random amount",
       "-swirl degrees       swirl image pixels about the center",
       "-texture filename    name of texture to tile onto the image background",
       "-threshold value     threshold the image",
       "-transparent color   make this color transparent within the image",
       "-treedepth value     depth of the color classification tree",
-      "-undercolor geometry control undercolor removal and black generation",
+      "-units type          PixelsPerInch, PixelsPerCentimeter, or Undefined",
       "-verbose             print detailed information about the image",
       "-view                FlashPix viewing transforms",
       "-wave geometry       alter an image along a sine wave",
       (char *) NULL
     };
 
-  (void) printf("Version: %s\n",Version);
-  (void) printf("Copyright: %s\n\n",Copyright);
+  (void) printf("Version: %.1024s\n",MagickVersion);
+  (void) printf("Copyright: %.1024s\n\n",MagickCopyright);
   (void) printf(
-    "Usage: %s [-options ...] file [ [-options ...] file ...]\n",client_name);
+    "Usage: %.1024s [-options ...] file [ [-options ...] file ...]\n",
+    client_name);
   (void) printf("\nWhere options include: \n");
   for (p=options; *p != (char *) NULL; p++)
-    (void) printf("  %s\n",*p);
+    (void) printf("  %.1024s\n",*p);
   (void) printf(
     "\nBy default, the image format of `file' is determined by its magic\n");
   (void) printf(
@@ -263,6 +279,8 @@ static void Usage(const char *client_name)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
+  ListMagickInfo(stdout);
+  ListDelegateInfo(stdout);
   Exit(0);
 }
 
@@ -287,6 +305,9 @@ int main(int argc,char **argv)
     *format,
     *option;
 
+  double
+    sans;
+
   ImageInfo
     image_info;
 
@@ -294,8 +315,12 @@ int main(int argc,char **argv)
     *image;
 
   int
+    gravity,
     x;
 
+  register Image
+    *p;
+
   register int
     i;
 
@@ -305,10 +330,11 @@ int main(int argc,char **argv)
     status;
 
   /*
-    Display usage profile if there are no command line arguments.
+    Initialize command line arguments.
   */
   ReadCommandlLine(argc,&argv);
   client_name=SetClientName(*argv);
+  (void) ExpandFilenames(&argc,&argv);
   if (argc < 2)
     Usage(client_name);
   /*
@@ -319,24 +345,46 @@ int main(int argc,char **argv)
   image=(Image *) NULL;
   GetImageInfo(&image_info);
   scene=0;
+  status=False;
   /*
     Parse command line.
   */
-  ExpandFilenames(&argc,&argv);
   for (i=1; i < argc; i++)
   {
     option=argv[i];
     if ((Extent(option) > 1) && ((*option == '-') || (*option == '+')))
       switch (*(option+1))
       {
+        case 'a':
+        {
+          if (strncmp("antialias",option+1,3) == 0)
+            {
+              image_info.antialias=(*option == '-');
+              break;
+            }
+          MagickError(OptionError,"Unrecognized option",option);
+          break;
+        }
         case 'b':
         {
+          if (strncmp("background",option+1,5) == 0)
+            {
+              image_info.background_color=(char *) NULL;
+              if (*option == '-')
+                {
+                  i++;
+                  if (i == argc)
+                    MagickError(OptionError,"Missing background color",option);
+                  (void) CloneString(&image_info.background_color,argv[i]);
+                }
+              break;
+            }
           if (strncmp("blur",option+1,3) == 0)
             {
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing factor",option);
                 }
               break;
@@ -353,11 +401,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("bordercolor",option+1,7) == 0)
             {
+              image_info.border_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing border color",option);
+                  (void) CloneString(&image_info.border_color,argv[i]);
                 }
               break;
             }
@@ -369,7 +419,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing box color",option);
-                  image_info.box=argv[i];
+                  (void) CloneString(&image_info.box,argv[i]);
                 }
               break;
             }
@@ -383,7 +433,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing factor",option);
                 }
               break;
@@ -425,12 +475,16 @@ int main(int argc,char **argv)
                     image_info.colorspace=OHTAColorspace;
                   if (Latin1Compare("rgb",option) == 0)
                     image_info.colorspace=RGBColorspace;
+                  if (Latin1Compare("srgb",option) == 0)
+                    image_info.colorspace=sRGBColorspace;
                   if (Latin1Compare("transparent",option) == 0)
                     image_info.colorspace=TransparentColorspace;
                   if (Latin1Compare("xyz",option) == 0)
                     image_info.colorspace=XYZColorspace;
                   if (Latin1Compare("ycbcr",option) == 0)
                     image_info.colorspace=YCbCrColorspace;
+                  if (Latin1Compare("ycc",option) == 0)
+                    image_info.colorspace=YCCColorspace;
                   if (Latin1Compare("yiq",option) == 0)
                     image_info.colorspace=YIQColorspace;
                   if (Latin1Compare("ypbpr",option) == 0)
@@ -464,6 +518,12 @@ int main(int argc,char **argv)
                   image_info.compression=UndefinedCompression;
                   if (Latin1Compare("None",option) == 0)
                     image_info.compression=NoCompression;
+                  if (Latin1Compare("BZip",option) == 0)
+                    image_info.compression=BZipCompression;
+                  if (Latin1Compare("Fax",option) == 0)
+                    image_info.compression=FaxCompression;
+                  if (Latin1Compare("Group4",option) == 0)
+                    image_info.compression=Group4Compression;
                   if (Latin1Compare("JPEG",option) == 0)
                     image_info.compression=JPEGCompression;
                   if (Latin1Compare("LZW",option) == 0)
@@ -524,13 +584,25 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info.density=argv[i];
+                  (void) CloneString(&image_info.density,argv[i]);
+                }
+              break;
+            }
+          if (strncmp("depth",option+1,3) == 0)
+            {
+              image_info.depth=QuantumDepth;
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    MagickError(OptionError,"Missing image depth",option);
+                  image_info.depth=atoi(argv[i]);
                 }
               break;
             }
           if (strncmp("despeckle",option+1,3) == 0)
             break;
-          if (strcmp("display",option+1) == 0)
+          if (Latin1Compare("display",option+1) == 0)
             {
               image_info.server_name=(char *) NULL;
               if (*option == '-')
@@ -538,7 +610,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing server name",option);
-                  image_info.server_name=argv[i];
+                  (void) CloneString(&image_info.server_name,argv[i]);
                 }
               break;
             }
@@ -579,7 +651,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing factor",option);
                 }
               break;
@@ -595,6 +667,53 @@ int main(int argc,char **argv)
         }
         case 'f':
         {
+          if (strncmp("filter",option+1,3) == 0)
+            {
+              if (*option == '-')
+                {
+                  FilterType
+                    filter;
+
+                  i++;
+                  if (i == argc)
+                    MagickError(OptionError,"Missing type",option);
+                  option=argv[i];
+                  filter=UndefinedFilter;
+                  if (Latin1Compare("Point",option) == 0)
+                    filter=PointFilter;
+                  if (Latin1Compare("Box",option) == 0)
+                    filter=BoxFilter;
+                  if (Latin1Compare("Triangle",option) == 0)
+                    filter=TriangleFilter;
+                  if (Latin1Compare("Hermite",option) == 0)
+                    filter=HermiteFilter;
+                  if (Latin1Compare("Hanning",option) == 0)
+                    filter=HanningFilter;
+                  if (Latin1Compare("Hamming",option) == 0)
+                    filter=HammingFilter;
+                  if (Latin1Compare("Blackman",option) == 0)
+                    filter=BlackmanFilter;
+                  if (Latin1Compare("Gaussian",option) == 0)
+                    filter=GaussianFilter;
+                  if (Latin1Compare("Quadratic",option) == 0)
+                    filter=QuadraticFilter;
+                  if (Latin1Compare("Cubic",option) == 0)
+                    filter=CubicFilter;
+                  if (Latin1Compare("Catrom",option) == 0)
+                    filter=CatromFilter;
+                  if (Latin1Compare("Mitchell",option) == 0)
+                    filter=MitchellFilter;
+                  if (Latin1Compare("Lanczos",option) == 0)
+                    filter=LanczosFilter;
+                  if (Latin1Compare("Bessel",option) == 0)
+                    filter=BesselFilter;
+                  if (Latin1Compare("Sinc",option) == 0)
+                    filter=SincFilter;
+                  if (filter == UndefinedFilter)
+                    MagickError(OptionError,"Invalid filter type",option);
+                }
+              break;
+            }
           if (strncmp("flip",option+1,3) == 0)
             break;
           if (strncmp("flop",option+1,3) == 0)
@@ -607,7 +726,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing font name",option);
-                  image_info.font=argv[i];
+                  (void) CloneString(&image_info.font,argv[i]);
                 }
               break;
             }
@@ -619,7 +738,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing image type",option);
-                  format=argv[i];
+                  (void) CloneString(&format,argv[i]);
                   (void) strcpy(image_info.filename,format);
                   (void) strcat(image_info.filename,":");
                   SetImageInfo(&image_info,False);
@@ -638,6 +757,18 @@ int main(int argc,char **argv)
                 }
               break;
             }
+          if (strncmp("fuzz",option+1,2) == 0)
+            {
+              image_info.fuzz=0;
+              if (*option == '-')
+                {
+                  i++;
+                  if ((i == argc) || !sscanf(argv[i],"%d",&x))
+                    MagickError(OptionError,"Missing distance",option);
+                  image_info.fuzz=atoi(argv[i]);
+                }
+              break;
+            }
           MagickError(OptionError,"Unrecognized option",option);
           break;
         }
@@ -646,7 +777,7 @@ int main(int argc,char **argv)
           if (strncmp("gamma",option+1,2) == 0)
             {
               i++;
-              if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+              if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                 MagickError(OptionError,"Missing value",option);
               break;
             }
@@ -660,6 +791,38 @@ int main(int argc,char **argv)
                 }
               break;
             }
+          if (strncmp("gravity",option+1,2) == 0)
+            {
+              gravity=ForgetGravity;
+              if (*option == '-')
+                {
+                  i++;
+                  if (i == argc)
+                    MagickError(OptionError,"Missing type",option);
+                  option=argv[i];
+                  if (Latin1Compare("NorthWest",option) == 0)
+                    gravity=NorthWestGravity;
+                  if (Latin1Compare("North",option) == 0)
+                    gravity=NorthGravity;
+                  if (Latin1Compare("NorthEast",option) == 0)
+                    gravity=NorthEastGravity;
+                  if (Latin1Compare("West",option) == 0)
+                    gravity=WestGravity;
+                  if (Latin1Compare("Center",option) == 0)
+                    gravity=CenterGravity;
+                  if (Latin1Compare("East",option) == 0)
+                    gravity=EastGravity;
+                  if (Latin1Compare("SouthWest",option) == 0)
+                    gravity=SouthWestGravity;
+                  if (Latin1Compare("South",option) == 0)
+                    gravity=SouthGravity;
+                  if (Latin1Compare("SouthEast",option) == 0)
+                    gravity=SouthEastGravity;
+                  if (gravity == ForgetGravity)
+                    MagickError(OptionError,"Invalid gravity type",option);
+                }
+              break;
+            }
           MagickError(OptionError,"Unrecognized option",option);
           break;
         }
@@ -680,7 +843,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing amount",option);
                 }
               break;
@@ -750,11 +913,13 @@ int main(int argc,char **argv)
             }
           if (strncmp("linewidth",option+1,2) == 0)
             {
+              image_info.linewidth=0;
               if (*option == '-')
                 {
                   i++;
                   if ((i == argc) || !sscanf(argv[i],"%d",&x))
                     MagickError(OptionError,"Missing size",option);
+                  image_info.linewidth=atoi(argv[i]);
                 }
               break;
             }
@@ -786,15 +951,17 @@ int main(int argc,char **argv)
                 }
               break;
             }
-          if (strcmp("matte",option+1) == 0)
+          if (Latin1Compare("matte",option+1) == 0)
             break;
           if (strncmp("mattecolor",option+1,6) == 0)
             {
+              image_info.matte_color=(char *) NULL;
               if (*option == '-')
                 {
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing matte color",option);
+                  (void) CloneString(&image_info.matte_color,argv[i]);
                 }
               break;
             }
@@ -803,7 +970,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing value",option);
                 }
               break;
@@ -828,12 +995,12 @@ int main(int argc,char **argv)
                   if (i == argc)
                     MagickError(OptionError,"Missing type",option);
                   option=argv[i];
-                  if ((Latin1Compare("uniform",option) != 0) &&
-                      (Latin1Compare("gaussian",option) != 0) &&
-                      (Latin1Compare("multiplicative",option) != 0) &&
-                      (Latin1Compare("impulse",option) != 0) &&
-                      (Latin1Compare("laplacian",option) != 0) &&
-                      (Latin1Compare("poisson",option) != 0))
+                  if ((Latin1Compare("Uniform",option) != 0) &&
+                      (Latin1Compare("Gaussian",option) != 0) &&
+                      (Latin1Compare("Multiplicative",option) != 0) &&
+                      (Latin1Compare("Impulse",option) != 0) &&
+                      (Latin1Compare("Laplacian",option) != 0) &&
+                      (Latin1Compare("Poisson",option) != 0))
                     MagickError(OptionError,"Invalid noise type",option);
                 }
               break;
@@ -890,13 +1057,13 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing pen color",option);
-                  image_info.pen=argv[i];
+                  (void) CloneString(&image_info.pen,argv[i]);
                 }
               break;
             }
           if (strncmp("pointsize",option+1,2) == 0)
             {
-              image_info.pointsize=atoi(DefaultPointSize);
+              image_info.pointsize=12;
               if (*option == '-')
                 {
                   i++;
@@ -913,7 +1080,7 @@ int main(int argc,char **argv)
         {
           if (strncmp("quality",option+1,2) == 0)
             {
-              image_info.quality=atoi(DefaultImageQuality);
+              image_info.quality=75;
               if (*option == '-')
                 {
                   i++;
@@ -1008,7 +1175,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing threshold",option);
                 }
               break;
@@ -1025,7 +1192,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing factor",option);
                 }
               break;
@@ -1035,7 +1202,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing geometry",option);
                 }
               break;
@@ -1048,7 +1215,7 @@ int main(int argc,char **argv)
                   i++;
                   if ((i == argc) || !IsGeometry(argv[i]))
                     MagickError(OptionError,"Missing geometry",option);
-                  image_info.size=argv[i];
+                  (void) CloneString(&image_info.size,argv[i]);
                 }
               break;
             }
@@ -1077,7 +1244,7 @@ int main(int argc,char **argv)
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
+                  if ((i == argc) || !sscanf(argv[i],"%lf",&sans))
                     MagickError(OptionError,"Missing degrees",option);
                 }
               break;
@@ -1095,7 +1262,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing filename",option);
-                  image_info.texture=argv[i];
+                  (void) CloneString(&image_info.texture,argv[i]);
                 }
               break;
             }
@@ -1134,16 +1301,20 @@ int main(int argc,char **argv)
         }
         case 'u':
         {
-          if (strncmp("undercolor",option+1,2) == 0)
+          if (strncmp("units",option+1,3) == 0)
             {
-              image_info.undercolor=(char *) NULL;
+              image_info.units=UndefinedResolution;
               if (*option == '-')
                 {
                   i++;
-                  if ((i == argc) || !sscanf(argv[i],"%f",(float *) &x))
-                    MagickError(OptionError,"Missing undercolor geometry",
-                      option);
-                  image_info.undercolor=argv[i];
+                  if (i == argc)
+                    MagickError(OptionError,"Missing type",option);
+                  option=argv[i];
+                  image_info.units=UndefinedResolution;
+                  if (Latin1Compare("PixelsPerInch",option) == 0)
+                    image_info.units=PixelsPerInchResolution;
+                  if (Latin1Compare("PixelsPerCentimeter",option) == 0)
+                    image_info.units=PixelsPerCentimeterResolution;
                 }
               break;
             }
@@ -1165,7 +1336,7 @@ int main(int argc,char **argv)
                   i++;
                   if (i == argc)
                     MagickError(OptionError,"Missing view transform",option);
-                  image_info.view=argv[i];
+                  (void) CloneString(&image_info.view,argv[i]);
                 }
               break;
             }
@@ -1174,7 +1345,7 @@ int main(int argc,char **argv)
         }
         case 'w':
         {
-          if (strcmp("wave",option+1) == 0)
+          if (Latin1Compare("wave",option+1) == 0)
             {
               i++;
               if ((i == argc) || !sscanf(argv[i],"%d",&x))
@@ -1203,10 +1374,12 @@ int main(int argc,char **argv)
         (void) strcpy(image_info.filename,argv[i]);
         image=ReadImage(&image_info);
         if (image == (Image *) NULL)
-          if (*option == '-')
-            break;
-          else
-            continue;
+          {
+            if (*option == '-')
+              break;
+            else
+              continue;
+          }
         if (scene != 0)
           image->scene=scene;
         if (format != (char *) NULL)
@@ -1218,14 +1391,14 @@ int main(int argc,char **argv)
               Modify filename to include a user specified image format.
             */
             p=image->filename+Extent(image->filename)-1;
-            while ((*p != '.') && (p > image->filename))
+            while ((*p != '.') && (p > (image->filename+1)))
               p--;
             p++;
-            if (strcmp(p,image->magick) == 0)
+            if (Latin1Compare(p,image->magick) == 0)
               (void) strcpy(p,format);
             else
               {
-                (void) sprintf(image_info.filename,"%s:%s",format,
+                FormatString(image_info.filename,"%.1024s:%.1024s",format,
                   image->filename);
                 (void) strcpy(image->filename,image_info.filename);
               }
@@ -1235,31 +1408,46 @@ int main(int argc,char **argv)
         */
         MogrifyImages(&image_info,i,argv,&image);
         if (global_colormap)
-          MapImages(image,(Image *) NULL,image_info.dither);
+          (void) MapImages(image,(Image *) NULL,image_info.dither);
         /*
-          Write transmogified image to disk.
+          Write transmogrified image to disk.
         */
-        if ((strcmp(image_info.filename,"-") == 0) || (format != (char *) NULL))
-          {
-            if (format != (char *) NULL)
-              AppendImageFormat(format,image->filename);
-            (void) WriteImage(&image_info,image);
-            DestroyImage(image);
-            continue;
-          }
-        (void) strcpy(filename,image->filename);
-        LocaleFilename(image->filename);
-        status=WriteImage(&image_info,image);
-        if (status == True)
-          {
-            (void) remove(filename);
-            (void) rename(image->filename,filename);
-          }
+        if (format != (char *) NULL)
+          AppendImageFormat(format,image->filename);
+        else
+          if (strcmp(image_info.filename,"-") != 0)
+            {
+              (void) strcpy(filename,image->filename);
+              LocaleFilename(image->filename);
+            }
+        for (p=image; p != (Image *) NULL; p=p->next)
+        {
+          (void) strcpy(p->filename,image->filename);
+          p->scene=scene++;
+        }
+        SetImageInfo(&image_info,True);
+        for (p=image; p != (Image *) NULL; p=p->next)
+        {
+          status=WriteImage(&image_info,p);
+          if ((status == False) || image_info.adjoin)
+            break;
+        }
+        for (p=image; p != (Image *) NULL; p=p->next)
+          scene--;
+        if (image_info.verbose)
+          DescribeImage(image,stdout,False);
+        if ((format == (char *) NULL) && (status != False))
+          if (Latin1Compare(image_info.filename,"-") != 0)
+            {
+              (void) remove(filename);
+              (void) rename(image->filename,filename);
+            }
         DestroyImage(image);
       }
     }
   if (image == (Image *) NULL)
     MagickError(OptionError,"Missing an image file name",(char *) NULL);
-  Exit(0);
+  DestroyDelegateInfo();
+  Exit(status ? 0 : errno);
   return(False);
 }
